package com.topcoder.web.urlhistory;

import com.topcoder.web.urlhistory.persistence.PersistenceException;
import com.topcoder.web.urlhistory.persistence.PersistenceStrategy;

/**
 * The GeneralManager class is the main entry point in this component.
 * This class has two functionalities: it handles the persistence
 * configuration details and it offers an access point to the other managers.
 * 
 * The persistence configuration handling simply means getting the name of
 * the persistence class from the configuration file and creating an
 * instance using reflection.
 * 
 * The reason for splitting the functionality into the 5 manager classes is
 * better grouping according to functionality. The 5 managers could have been
 * just as well one big class but it would have had too many methods.
 * Having separate classes for each group of functionalities is cleaner. 
 * 
 * This class is a singleton. Since the component is user based (as keeping
 * information is concerned) there isn't a good reason to have more instances
 * of this class. This class is also a facade for the user option management
 * functionality.
 */
public class GeneralManager {
    
    /**
     * The Configuration Manager namespace used by this component.
     */
    public static final String NAMESPACE = "com.topcoder.web.urlhistory";

    /**
     * The default config manager file.
     * This file is loaded only if the config manager namespace is not created
     * when the component is instantiated for the first time. This gives the
     * flexibility to use other config manager file locations or formats,
     * essential in development of applications using many different components,
     * with different configuration files, to avoid the nightmare of having a
     * lot of different config files, spread everywhere.
     */
    private static final String DEFAULT_CONFIG_FILE = "conf\\URLHistory.properties";

    /**
     * Constant for the property specifying the persistence class.
     */
    private static final String PERSISTENCE_CLASS_PROPERTY = "persistence_class";
    
    /**
     * The singleton instance of this class. Created in the getInstance method.
     */
    private static GeneralManager instance = null;
    
    /**
     * The persistence instance. Created in the constructor using reflection.
     */
    private PersistenceStrategy persistence = null;
    
    /**
     * The user manager instance. Created in the constructor.
     */
    private UserManager userManager = null;
    
    /**
     * The group manager instance. Created in the constructor.
     */
    private GroupManager groupManager = null;
    
    /**
     * The URL manager instance. Created in the constructor.
     */
    private URLManager urlManager = null;
    
    /**
     * The statistics manager instance. Created in the constructor.
     */
    private StatisticsManager statisticsManager = null;

    /**
     * Private constructor that prevents outside instantiation (singleton).
     *
     * The default config manager file is loaded only if the config manager
     * namespace is not created when the component is instantiated for the first
     * time. This gives the flexibility to use other config manager file
     * locations or formats, essential in development of applications using many
     * different components, with different configuration files, to avoid the
     * nightmare of having a lot of different config files, spread everywhere.
     *
     * This class reads the persistence_class property and creates an
     * instance of the given class using reflection (the constructor is
     * dinamically invoked, passing the config namespace as parameter).
     *
     * @throws ConfigurationException used to wrap any exceptions related to the
     * configuration data or that indicate a problem with the configuration file:
     * configuration manager exceptions indicating bad or missing file, missing
     * persistence_class property, reflection exceptions indicating bad class name
     * @throws PersistenceException wraps an exception coming from the persistence
     * constructor indicated some problem with the initialization of the persistence
     */
    private GeneralManager() throws ConfigurationException, PersistenceException {
    }

    /**
     * Gets the singleton instance (singleton pattern).
     *
     * @return the singleton instance of this class
     *
     * @throws ConfigurationException used to wrap any exceptions related to the
     * configuration data or that indicate a problem with the configuration file:
     * configuration manager exceptions indicating bad or missing file, missing
     * persistence_class property, reflection exceptions indicating bad class name
     * @throws PersistenceException wraps an exception coming from the persistence
     * constructor indicated some problem with the initialization of the persistence
     */
    public static GeneralManager getInstance() throws ConfigurationException, PersistenceException {
        return null;
    }
    
    /**
     * Getter for the persistenceStrategy field.
     * 
     * @return the value of the persistenceStrategy field
     */
    public PersistenceStrategy getPersistence() {
        return null;
    }
    
    /**
     * Getter for the userManager field.
     *
     * @return returns the value of userManager field
     */
    public UserManager getUserManager() {
        return null;
    }
    
    /**
     * Getter for the groupManager field.
     *
     * @return returns the value of groupManager field
     */
    public GroupManager getGroupManager() {
        return null;
    }
    
    /**
     * Getter for the urlManager field.
     *
     * @return returns the value of urlManager field
     */
    public URLManager getURLManager() {
        return null;
    }
    
    /**
     * Getter for the statisticsManager field.
     *
     * @return returns the value of statisticsManager field
     */
    public StatisticsManager getStatisticsManager() {
        return null;
    }
    
}
