package com.topcoder.web.urlhistory;

import java.util.Set;

import com.topcoder.user.profile.UserProfile;
import com.topcoder.web.urlhistory.data.GroupInfo;
import com.topcoder.web.urlhistory.persistence.PersistenceException;

/**
 * The GroupManager class is a facade that provides group management functionality.
 * This class offers functionality to create groups and subgroups, updating
 * groups after making changes, removing groups and getting groups.
 * 
 * In order to transfer group data to and from the user, and to and from the
 * persistence layer, the GroupInfo class is used.
 * 
 * This class simply calls the persistence layer to perform the actual work. 
 */
public class GroupManager {
    
    /** 
     * Package private constructor. Defined only to prevent the user
     * from creating instances directly.
     */
    GroupManager() {        
    }
    
    /**
     * Creates a group (a root group).
     * 
     * This is accomplished by calling the createGroup(user, group, null) persistence method.
     * The group information, name, user defined attributes, never expire flag, 
     * black list flag are all embedded in the GroupInfo instance. Note that the id
     * of the group is generated by the persistence implementation (since it can be
     * generated implementation specific - using ID Generating or autoincrement fields
     * for example).
     * 
     * The only sanity check is whether the group isn't already persisted (must have zero id).
     *
     * @param user the user to create the group for
     * @param group the group to create
     *
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void createGroup(UserProfile user, GroupInfo group) throws NullPointerException, PersistenceException, InvalidDataException {
    }
    
    /**
     * Creates a subgroup in a group.
     * 
     * This is accomplished by calling the createGroup(null, group, parent) persistence method.
     * The group information, name, user defined attributes, never expire flag, 
     * black list flag are all embedded in the GroupInfo instance. Note that the id
     * of the group is generated by the persistence implementation (since it can be
     * generated implementation specific - using ID Generating or autoincrement fields
     * for example).
     * 
     * Before anything else, some sanity checks are being performed. The parent group
     * must already be created (must have non-zero id) and the child group must not
     * have been already created (zero id).
     * 
     * Note that the UserProfile isn't necessary here since the parent group already
     * offers user information (the user id).
     * 
     * @param group the child group to create
     * @param parent the parent group
     *
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void createGroup(GroupInfo group, GroupInfo parent) throws NullPointerException, PersistenceException, InvalidDataException {
    }
    
    /**
     * Updates a group.
     * 
     * This is accomplished by calling the updateGroup(group) persistence method.
     * The new group information, name, user defined attributes, never expire flag, 
     * black list flag are all embedded in the GroupInfo instance.
     * 
     * The only sanity check is whether the group is already persisted (must have
     * non-zero id).
     * 
     * Note that the UserProfile isn't necessary here since the group can be updated
     * using its id.
     * 
     * @param group the group
     *
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void updateGroup(GroupInfo group) throws NullPointerException, PersistenceException, InvalidDataException {        
    }
    
    /**
     * Removes a group or subgroup.
     * 
     * This is accomplished by calling the removeGroup(group) persistence method.
     * The group is given by its GroupInfo, but normally only the id would be necessary
     * in a normal persistence implementation to do the job.
     * 
     * Note that the children groups of this group are removed as well (recursively).
     * This is accomplished by getting the subgroups using the getSubgroups method
     * and calling this method recursively for each child. If the child have subgroups
     * they will be deleted in a similar manner. The owned URLs are deleted too.
     * They are obtained using the readURLs persistence method and deleted one by one
     * using the deleteURL method.
     * 
     * The only sanity check is whether the group is already persisted (must have
     * non-zero id).
     *
     * Note that the UserProfile isn't necessary here since the group can be removed
     * using its id.
     * 
     * @param group the group
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void removeGroup(GroupInfo group) throws NullPointerException, PersistenceException, InvalidDataException {
    }
    
    /**
     * Clears all the subgroups of a group.
     * 
     * This is accomplished by getting the list of subgroups using the readSubgroups(group) 
     * persistence method and deleting the subgroups one by one using the removeGroup
     * method.
     * 
     * Null group means the root groups should be deleted.
     * 
     * Note that subgroups are deleted including children (subgroups, URLs) by using
     * the removeGroup method defined in this class.
     *
     * The only sanity check is whether the group is already persisted (must have
     * non-zero id).
     * 
     * @param group the group (null allowed)
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void clearSubgroups(GroupInfo group) throws PersistenceException, InvalidDataException {
    }
    
    /**
     * Gets a group given its id.
     * 
     * This is accomplished by calling the readGroup(id) method.
     * 
     * @param id the group id
     * 
     * @return the group with the given id or null if none exists
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public GroupInfo getGroup(long id) throws PersistenceException {
        return null;
    }
        
    /**
     * Gets the parent group of a group.
     * 
     * This is accomplished by calling the readGroup(group.getParentId()) method.
     * 
     * The only sanity check is whether the group is already persisted (must have
     * non-zero id).
     *
     * @param group the group
     * 
     * @return the parent group or null if it is the root group (that
     * is indicated by a zero group.getParentId())
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public GroupInfo getParentGroup(GroupInfo group) throws NullPointerException, PersistenceException, InvalidDataException {
        return null;
    }
    
    /**
     * Gets the set of subgroups of a group. Null parent means root groups should
     * be retrieved.
     * 
     * This is accomplished using the readSubgroups(parent) persistence method.
     * 
     * The only sanity check is whether the group is already persisted (must have
     * non-zero id).
     * 
     * @param parent the parent group to find the subgroups for (null allowed)
     *
     * @return the set of subgroups (zero length is no subgroups exist)
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public Set getSubgroups(GroupInfo parent) throws PersistenceException, InvalidDataException {
        return null;
    }

}
