package com.topcoder.web.urlhistory;

import java.util.Comparator;
import java.util.Date;
import java.util.List;

import com.topcoder.user.profile.UserProfile;
import com.topcoder.web.urlhistory.compare.CompareStrategy;
import com.topcoder.web.urlhistory.data.GroupInfo;
import com.topcoder.web.urlhistory.data.URLInfo;
import com.topcoder.web.urlhistory.data.URLStatistics;
import com.topcoder.web.urlhistory.persistence.PersistenceException;

/**
 * The StatisticsManager class is a facade that provides statistics functionality.
 * This class offers functionality to get the visitation count, last visitation.
 * 
 * In order to transfer group data to and from the user, and to and from the
 * persistence layer, the GroupInfo class is used.
 * 
 * This class simply calls the persistence layer to perform the actual work. 
 */
public class StatisticsManager {

    /** 
     * Package private constructor. Defined only to prevent the user
     * from creating instances directly.
     */
    StatisticsManager() {        
    }
    
    /**
     * Called by the user to communicate that a URL has been visited.
     * The given input is the user making the visit, the URL string and
     * the date of the visit. 
     * 
     * This method tries to get the URLInfo for the given URL using 
     * readURL(UserProfile user, String url). The readURL method
     * automatically creates a URL entry if one doesn't exist. This
     * behavior is designer to allow the user to visit URLs without
     * having to create URLInfo instances first. The first visit
     * automatically creates one.
     * 
     * This method then simply calls the createVisit(urlInfo, date) persistence method.
     *
     * @param user the user
     * @param url the URL string
     * @param date the data of the visit
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public void visit(UserProfile user, String url, Date date) throws NullPointerException, PersistenceException {
    }
    
    /**
     * Called by the user to communicate that a URL has been visited.
     * The given input is the URLInfo of the URL (that happens to contain the 
     * user id) and the date of the visit. 
     * 
     * This method simply calls the createVisit(url, date) persistence method.
     * 
     * The sanity check is that the URLInfo must have been written to persistence
     * (non-zero id).  
     *
     * @param url the URL string
     * @param date the data of the visit
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void visit(URLInfo url, Date date) throws NullPointerException, PersistenceException, InvalidDataException {        
    }
    
    /** 
     * Gets the visit statistics for a URL. This method is quick because it only looks in the
     * history of the given URL, as opposite to the other overload that retrieves
     * all the URLs for the user, so it can check equality using the comparator.
     * 
     * This method simply calls getVisitStatistic(url). The returned object
     * exposes the visitation count and the last visitation date of the URL, as
     * asked by the requirements.
     * 
     * Note that since this method gets the information about a specific URL,
     * it doesn't take into account whether it is black listed. That is because
     * the result would be invariably 0 in case of black listed URLs, making
     * the call useless.
     * 
     * The given URLInfo must be previously persisted (non-zero id) (sanity check). 
     *
     * @param url the URLInfo of the URL
     * 
     * @return the visit statistics (including visit count and last visit) for the given URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public URLStatistics getVisitStatistics(URLInfo url) throws NullPointerException, PersistenceException, InvalidDataException {
        return null;
    }
    
    /** 
     * Gets the visit statistics for a URL string. The returned result aggregate
     * the statistics for all URLs that match the given URL string according to
     * the given comparator. 
     * 
     * This method retrieves all URLs for the given user using getVisitStatistics(user).
     * 
     * Then it iterates the URLs and selects only those that are equal to url
     * according to the comparator (areEqual(url, item.getURL() is true) and
     * that are not black listed (isBlackListed(user, urlStatistics) is false).
     * 
     * The information from the selected URLStatistics instances are aggregated into
     * one URLStatistics object and returned. The URL strings are aggregated using
     * the getCommonPart method of the comparator. The visit count uses sum.
     * The last visit uses max date. The personal rating uses average.
     * 
     * The returned object exposes the visitation count and the last visitation 
     * date of the URL, as asked by the requirements. 
     *
     * @param user the user
     * @param url the URL string
     * @param comparator the comparator instance
     * 
     * @return the visit statistics (including visit count and last visit) for the given URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public URLStatistics getVisitStatistics(UserProfile user, String url, CompareStrategy comparator) throws NullPointerException, PersistenceException {
        return null;
    }

    /** 
     * Gets the visit statistics for the URLs for a user.
     * 
     * Before returning the statistics the list suffers two transformations.
     * The first one eliminates all duplicate URLs (according to the comparator)
     * by aggregating their statistics data into one URLStatistics. The second
     * one simply sorts the list using the given sorting criteria. 
     *  
     * 1. Retrieves all URLs for the given user using getVisitStatistics(user).
     * 
     * 2. Filter those that are black listed (isBlackListed(user, urlStatistics) is true).
     * 
     * 3. Detect duplicates in O(n) by using a HashMap in which the getCompareKey method
     *    of the comparator provides the keys for each URLStatistics. By putting
     *    the keys one by one in the map, duplicates are easily found, because
     *    the getCompareKey method guarantees that equal keys imply duplicates.
     * 
     * 4. Each group of duplicates detected in the previous step are aggregated into
     *    one URLStatistics object. The URL strings are aggregated using
     *    the getCommonPart method of the comparator. The visit count uses sum.
     *    The last visit uses max date. The personal rating uses average.
     * 
     * 5. The list of URLStatistics object is finally sorted using
     *    Collections.sort(list, comparator).
     * 
     * Then it iterates the URLs and selects only those that are equal to url
     * according to the comparator (areEqual(url, item.getURL() is true).
     * 
     * The information from the selected URLStatistics instances are aggregated into
     * one URLStatistics object and returned. The URL strings are aggregated using
     * the getCommonPart method of the comparator. The visit count uses sum.
     * The last visit uses max date. The personal rating uses average.
     * 
     * The returned object exposes the visitation count and the last visitation 
     * date of the URL, as asked by the requirements. 
     *
     * @param user the user
     * @param sorter the sorter instance
     * @param comparator the comparator instance
     * 
     * @return the visit statistics (including visit count and last visit) for the given URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public List sortURLs(UserProfile user, Comparator sorter, CompareStrategy comparator) throws NullPointerException, PersistenceException {
        return null;
    }
    
    /** 
     * Gets the visit statistics for the groups for a user.
     * 
     * Before returning the statistics the list it is sorted using the given 
     * sorting criteria. 
     *  
     * 1. Retrieves all URLs for the given user using getVisitStatistics(user).
     * 
     * 2. Filter those that are black listed (isBlackListed(user, urlStatistics) is true). 
     * 
     * 3. Use the group names from each URLStatistics object to obtain a mapping from
     *    group names to lists of URLStatistics belonging to them. If a URL belongs
     *    to more groups, then it will appear in the list of each group. 
     * 
     * 4. Each of the entries in the map from step 2 result in one GroupStatistics
     *    object that aggregates the URLStatistics objects. The aggregation is
     *    done as above. The visit count uses sum. The last visit uses max date. 
     *    The personal rating uses average.
     * 
     * 5. The list of URLStatistics object is finally sorted using
     *    Collections.sort(list, comparator).
     * 
     * Then it iterates the URLs and selects only those that are equal to url
     * according to the comparator (areEqual(url, item.getURL() is true).
     * 
     * The information from the selected URLStatistics instances are aggregated into
     * one URLStatistics object and returned. The URL strings are aggregated using
     * the getCommonPart method of the comparator. The visit count uses sum.
     * The last visit uses max date. The personal rating uses average.
     * 
     * The returned object exposes the visitation count and the last visitation 
     * date of the URL, as asked by the requirements. 
     *
     * @param user the user
     * @param sorter the sorter instance
     * 
     * @return the visit statistics (including visit count and last visit) for the given URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public List sortGroups(UserProfile user, Comparator sorter) throws NullPointerException, PersistenceException {
        return null;
    }
    
    /**
     * Finds whether the given URLStatistics object is black listed or not,
     * directly or as a side-effect of belonging to a group.
     * 
     * 1. The URLInfo object is retrieved using readURL(user, url.getUrl()).
     * 
     * 2. If the black list flag of the URLInfo is either YES or NO, true and false
     *    is immediately returned.
     *  
     * 3. Otherwise the parent groups of the URL are iterated and their black listed
     *    status is determined using the isBlackListed method defined below.
     * 
     * 4. One parent group that is black listed is enough to return true.
     *    (even if there are parents that aren't black listed - because one black 
     *    listed group is enough to make the URL blacklisted according to the 
     *    requirements).
     * 
     * 5. If no black listed parents exists then false is returned.   
     *
     * @param user the user
     * @param url the URLStatistics object
     * 
     * @return whether the URL is black listed
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    boolean isBlackListed(UserProfile user, URLStatistics url) throws NullPointerException, PersistenceException {
        return false;
    }
    
    /**
     * Finds whether the given GroupInfo object is black listed or not,
     * directly or as a side-effect of being a subgroup.
     * 
     * 1. If the group is marked with YES or NO then true or false is returned.
     * 2. If not, readGroup(group.getParentId()) is used to find the parent group
     *    and step 1 is repeated with the parent as the group.
     * 3. If there is no parent group or all parents have been iterated down
     *    to the root, then false is returned.
     * 
     * @param group the group
     * 
     * @return whether the group is black listed
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    boolean isBlackListed(GroupInfo group) throws NullPointerException, PersistenceException {
        return false;        
    }
    
}
