package com.topcoder.web.urlhistory;

import java.util.Set;

import com.topcoder.user.profile.UserProfile;
import com.topcoder.web.urlhistory.compare.CompareStrategy;
import com.topcoder.web.urlhistory.data.Flag;
import com.topcoder.web.urlhistory.data.GroupInfo;
import com.topcoder.web.urlhistory.data.URLInfo;
import com.topcoder.web.urlhistory.persistence.PersistenceException;

/**
 * The URLManager class is a facade that provides URL management functionality.
 * This class offers functionality to create URLs, to update them after changing
 * their attributes, to remove URLs and getting URLs.
 * 
 * In order to transfer URL data to and from the user, and to and from the
 * persistence layer, the URLInfo class is used.
 * 
 * This class simply calls the persistence layer to perform the actual work. 
 */
public class URLManager {

    /** 
     * Package private constructor. Defined only to prevent the user
     * from creating instances directly.
     */
    URLManager() {        
    }
    
    /**
     * Creates a URL.
     * 
     * This is accomplished by calling the createURL(user, url) persistence method.
     * The URL information, url, name, user defined attributes, never expire flag, 
     * black list flag, parent groups are all embedded in the GroupInfo instance. 
     * Note that the id of the group is generated by the persistence implementation 
     * (since it can be generated implementation specific - using ID Generating or 
     * autoincrement fields for example).
     * 
     * The sanity checks are whether the URL is already persisted (must have zero id)
     * and the owner of each of the parent groups (specified in the URLInfo instance)
     * must be the same as the given user. The readUser persistence method can be
     * used to find the user id given the user profile. Note that readUser creates
     * the user entry in persistence if none exists yet (to allow easier usage by not
     * having to manually create users).
     *
     * @param user the user to create the group for
     * @param url the URL to create
     *
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void createURL(UserProfile user, URLInfo url) throws NullPointerException, PersistenceException, InvalidDataException {
    }
    
    /**
     * Updates a URL.
     * 
     * This is accomplished by calling the updateURL(url) persistence method.
     * The URL information, url, name, user defined attributes, never expire flag, 
     * black list flag, parent groups are all embedded in the GroupInfo instance. 
     * 
     * The sanity checks are whether the URL isn't already persisted (must have non-zero id)
     * and the owner of each of the parent groups (specified in the URLInfo instance)
     * must be the same as the owner of the URL (done by comparing the user ids).
     * 
     * Note that the user is not needed here since the id of the URL should be enough,
     * plus that the userId is stored in the URL also.
     *
     * @param url the URL to create
     *
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void updateURL(URLInfo url) throws NullPointerException, PersistenceException, InvalidDataException {        
    }
    
    /**
     * Removes a URL.
     * 
     * This is accomplished by calling the removeURL(url) persistence method.
     * The group is given by its URLInfo, but normally only the id would be necessary
     * in a normal persistence implementation to do the job.
     * 
     * The only sanity check is whether the URL is already persisted (must have
     * non-zero id).
     *
     * Note that the UserProfile isn't necessary here since the URL can be removed
     * using its id.
     * 
     * @param url the URL to create
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void removeURL(URLInfo url) throws NullPointerException, PersistenceException, InvalidDataException {
    }
    
    /**
     * Clears all the URLs from a group.
     * 
     * This is accomplished by getting the list of URLs using the readURLs(group) 
     * persistence method and deleting the URLs one by one using the removeURL
     * method.
     * 
     * Null group means the root URLs (belonging to no group) should be deleted.
     *
     * The only sanity check is whether the group is already persisted (must have
     * non-zero id).
     * 
     * @param group the group (null allowed)
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void clearURLs(GroupInfo group) throws PersistenceException, InvalidDataException {
    }
    
    /**
     * Gets a URL given its id.
     * 
     * This is accomplished by calling the readURL(id) persistence method.
     * 
     * @param id the URL id
     * 
     * @return the URL with the given id or null if none exists
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public URLInfo getURL(long id) throws PersistenceException {
        return null;
    }
    
    /**
     * Gets a URL given its URL text.
     * 
     * This is accomplished by calling the readURL(user, url) method.
     * 
     * The readURL method automatically creates a URL entry if one doesn't exist. 
     * This behavior is designer to allow the user to visit URLs without
     * having to create URLInfo instances first. The first visit
     * automatically creates one.
     * 
     * @param user the user
     * @param url the URL string
     * 
     * @return the URL with the given text or null if none exists
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public URLInfo getURL(UserProfile user, String url) throws NullPointerException, PersistenceException {
        return null;
    }
    
    /**
     * Gets the set of URLs belonging to a group.
     * 
     * This is accomplished using the readURLs(group) persistence method.
     * 
     * The only sanity check is whether the group is already persisted (must have
     * non-zero id).
     * 
     * @param group the group to find the URLs for
     *
     * @return the set of URLs (zero length is no URLs exist in the given group)
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public Set getURLs(GroupInfo group) throws NullPointerException, PersistenceException, InvalidDataException {
        return null;
    }

    /**
     * Gets the set of URLs belonging to a user.
     * 
     * This is accomplished using the readAllURLs(user) persistence method.
     * 
     * @param user the user to find the URLs for
     *
     * @return the set of URLs (zero length is no URLs exist for the given user)
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public Set getAllURLs(UserProfile user) throws NullPointerException, PersistenceException {
        return null;
    }
    
    /**
     * Set the black listed flag for one or more URLs, depending on the comparator.
     * This method takes all the URLs for the user (got using the readAllURLs method),
     * iterates them, and sets the black listed flag for those that are equal to the
     * given URL, according to the comparer.
     * 
     * After the flag is set in an individual URL, the URL is updated in persistent storage
     * with updateURL.  
     *
     * @param user the user
     * @param url the URL string
     * @param comparator the comparator used to find the URLs to be black listed
     * @param blackListed the black list flag
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public void setBlackListed(UserProfile user, String url, CompareStrategy comparator, Flag blackListed) throws NullPointerException, PersistenceException {
    }
    
    /**
     * Set the never expire flag for one or more URLs, depending on the comparator.
     * This method takes all the URLs for the user (got using the readAllURLs method),
     * iterates them, and sets the never expire flag for those that are equal to the
     * given URL, according to the comparer.
     * 
     * After the flag is set in an individual URL, the URL is updated in persistent storage
     * with updateURL.  
     *
     * @param user the user
     * @param url the URL string
     * @param comparator the comparator used to find the URLs to be never expired
     * @param neverExpire the never expire flag
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public void setNeverExpire(UserProfile user, String url, CompareStrategy comparator, Flag neverExpire) throws NullPointerException, PersistenceException {
    }    
    
}
