package com.topcoder.web.urlhistory;

import java.util.Set;

import com.topcoder.user.profile.UserProfile;
import com.topcoder.web.urlhistory.data.UserInfo;
import com.topcoder.web.urlhistory.persistence.PersistenceException;

/**
 * The UserManager class is a facade for the user management functionality.
 * 
 * This class offers functionality to create users, update them, remove them
 * and get them.
 * 
 * The user management functionality is useful only for defining user specific
 * options, because besides that, users don't have any attributes in this
 * application. But in order to allow separate expiration options, the
 * users have been enriched with user defined options. These allow setting
 * the expiration options but also provide support for options that may
 * be needed by future functionality.
 * 
 * This class simply calls the persistence layer to perform the actual work.
 */
public class UserManager {

    /** 
     * Package private constructor. Defined only to prevent the user
     * from creating instances directly.
     */
    UserManager() {        
    }
    
    /**
     * Creates a user.
     * 
     * This is accomplished by calling the createUser(userProfile, user) persistence method.
     * The user information (the user defined attributes that allow use specific options)
     * are embedded in the UserInfo instance.
     *  
     * Note that the id of the user is generated by the persistence implementation 
     * (since it can be generated implementation specific - using ID Generating or 
     * autoincrement fields for example).
     * 
     * The sanity checks is whether the user isn't already persisted (must have zero id).
     *
     * @param userProfile the user profile of the user
     * @param user the user information (contains only user defined attributes)
     *
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void createUser(UserProfile userProfile, UserInfo user) throws NullPointerException, PersistenceException, InvalidDataException {
    }
    
    /**
     * Updates a user.
     * 
     * This is accomplished by calling the updateUser(user) persistence method.
     * The user information (the user defined attributes that allow use specific options)
     * are embedded in the UserInfo instance.
     *  
     * The sanity checks is whether the user is already persisted (must have non-zero id).
     *
     * Note that the UserProfile isn't necessary here since the user can be updated
     * using its id.
     * 
     * @param user the user information (contains only user defined attributes)
     *
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void updateUser(UserInfo user) throws NullPointerException, PersistenceException, InvalidDataException {
    }   

    /**
     * Removes a user.
     * 
     * This is accomplished by calling the removeUser(user) persistence method.
     * The group is given by its UserInfo, but normally only the id would be necessary
     * in a normal persistence implementation to do the job.
     * 
     * The only sanity check is whether the user is already persisted (must have
     * non-zero id).
     *
     * Note that the UserProfile isn't necessary here since the user can be removed
     * using its id.
     * 
     * @param user the user to remove
     * 
     * @throws NullPointerException if the argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     * @throws InvalidDataException if the request is not valid according to the sanity
     * check described above 
     */
    public void removeUser(UserInfo user) throws NullPointerException, PersistenceException, InvalidDataException {
    }    
        
    /**
     * Clears all the users.
     * 
     * This is accomplished by getting the list of users using the getAllUsers 
     * method and deleting the URLs one by one using the removeUser method.
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public void clearUsers() throws PersistenceException {
    }
    
    /**
     * Gets a user information object given its user profile.
     * 
     * This is accomplished by calling the readUser(user) persistence method.
     * Note that the readUser method automatically creates a user entry if none exists.
     * The purpose of this behavior is to allow the user to work with
     * groups and URLs without having to worry about user management,
     * by automatically creating user entries when needed.
     *  
     * @param user the user profile
     * 
     * @return the user information object with the given text
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public UserInfo getUser(UserProfile user) throws NullPointerException, PersistenceException {
        return null;
    }
    
    /**
     * Gets all user information objects.
     * 
     * This is accomplished by calling the readAllUser() persistence method.
     *  
     * @return the list with all users
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception)
     */
    public Set getAllUsers() throws PersistenceException {
        return null;
    }

}
