package com.topcoder.web.urlhistory.compare;

import java.util.Set;

/**
 * The CompareStrategy interface abstracts the URL compare logic.
 * This interface defines three methods: areEqual for determining
 * whether two URL strings are equal, getCommonPart to get the
 * common string part of a set of URL strings and getCompareKey
 * to achieve an efficient duplicate elimination algorithm for
 * the statistics.
 * 
 * The getCompareKey method is used to achieve an O(n) duplicate
 * elimination algorithm. The idea is to get from the URLs
 * keys (substrings) that are equal for duplicate URLs.
 * 
 * Putting these keys in a HashMap allows duplicate detection in
 * n*O(1) == O(n). Alternative algorithms could achieve O(nlogn) if
 * we use sorting on the keys. The trivial algorithm achieves O(n^2)
 * by comparing all possible pairs of URLs, but that is not acceptable.  
 */
public interface CompareStrategy {
    
    /**
     * The areEqual method returns whether the two URL strings are equal
     * according to this comparator.
     * 
     * Note that if the arguments are not formatted in the way the comparator
     * expects to (are not valid URLs), false is returned. No exception
     * is thrown.
     *
     * @param url1 the first URL string
     * @param url2 the second URL string
     * 
     * @return whether the URLs are equal
     * 
     * @throws NullPointerException if any of the arguments is null
     */
    public boolean areEqual(String url1, String url2) throws NullPointerException;
    
    /**
     * Returns a substring from the URL that can be used as key for duplicate
     * detection as described in the interface documentation.
     * 
     * Note that if the arguments is not formatted in the way the comparator
     * expects to (are not valid URLs), the full string is returned. No exception
     * is thrown.
     *
     * @param url the URL
     * @return a substring from the URL that can be used as key for duplicate
     * detection
     * @throws NullPointerException if the argument is null
     */
    public String getCompareKey(String url) throws NullPointerException;
    
    /**
     * Gets the common substring of a given set of URLs that can be used for
     * statistics / reporting purposes.
     * 
     * Note that if the arguments is not formatted in the way the comparator
     * expects to (are not valid URLs), an empty string is returned. No exception
     * is thrown. If two URLs are not equal, the empty string is returned also
     * since it means they don't have anything in common. 
     *
     * @param urls the set of URLs (strings)
     * 
     * @return the common substring
     * 
     * @throws NullPointerException if the argument is null
     * @throws IllegalArgumentException if the set contains something else
     * than Strings (including nulls)
     */
    public String getCommonPart(Set urls) throws NullPointerException, IllegalArgumentException;

}
