package com.topcoder.web.urlhistory.compare;

import java.util.Set;

/**
 * CompareStrategy implementation that uses text comparison
 * as criteria but only up to a specified level.
 * 
 * By level it is understood a substring (including the empty string)
 * surrounded by the / character, the host name being level 0.
 * 
 * This type of comparison compares a specified number of levels.
 * Everything else is ignored, including the protocol and the port.  
 */
public class PartialComparator implements CompareStrategy {
    
    /**
     * The level depth to compare.
     * 0 means root level (only the host name is compared).
     */
    private int levelDepth = 0;

    /**
     * Is the comparison case sensitive?
     */
    private boolean caseSensitive = false;
    
    /** 
     * Constructor.
     *
     * @param levelDepth the level depth
     * @param caseSensitive case sensitive comparison?
     * 
     * @throws IllegalArgumentException if levelDepth is negative
     */
    public PartialComparator(int levelDepth, boolean caseSensitive) throws IllegalArgumentException {        
    }
    
    /**
     * Return the level depth.
     *
     * @return the level depth
     */
    public int getLevelDepth() {
        return levelDepth;
    }
    
    /** 
     * Returns whether the comparison is case sensitive.
     *
     * @return whether the comparison is case sensitive
     */
    public boolean isCaseSensitive() {
        return false;
    }
    
    /**
     * The areEqual method returns whether the two URL strings are equal.
     * 
     * 1. Use two indices i and j to walk the characters of the URLs.
     * 
     * 2. Skip until the :// sequence is reached in both URLs.
     * 
     * 3. While chars are equal (considering the case sensitivity setting),
     *    advance in both URLs. If not equal or one string reaches end
     *    of string or end of level then return false.
     *    (if current level is 0 then both : and / indicate end of level)
     * 
     * 4. While current level less than or equal to levelDepth repeat steps
     *    3 and 4. 
     *
     * @param url1 the first URL string
     * @param url2 the second URL string
     * 
     * @return whether the URLs are equal
     * 
     * @throws NullPointerException if any of the arguments is null
     */
    public boolean areEqual(String url1, String url2) throws NullPointerException {
        return false;
    }
    
    /**
     * Returns a substring from the URL that can be used as key for duplicate
     * detection as described in the interface documentation.
     * 
     * 1. Use an index i to walk the characters of the URL.
     * 
     * 2. Skip until the :// sequence is reached in both URLs.
     * 
     * 3. Start copying chars in the output (lowercased if case insensitive). 
     *    Each / char increases the current level by one. When reaching 
     *    depthLevel return the output. 
     * 
     * 4. If level is 0 and : is encountered then skip everything till the /
     *    character of end or URL. 
     *
     * @param url the URL
     * @return a substring from the URL that can be used as key for duplicate
     * detection
     * @throws NullPointerException if the argument is null
     */
    public String getCompareKey(String url) throws NullPointerException {
        return null;
    }
    
    /**
     * Gets the common substring of a given set of URLs that can be used for
     * statistics / reporting purposes.
     * 
     * Iterates the URLs and builds the common part of 1 and 2, then the
     * result with 3, then the new result with 4 and so on.
     * 
     * Getting the common part of two URLs:
     * 
     * 1. Use two indices i and j to walk the characters of the URLs.
     * 
     * 2. Skip until the :// sequence is reached in both URLs.
     *    If the skipped chars are the same in the two URLs then they go
     *    to the output (case sensitivity is considered - lowercase chars
     *    are outputed if case insensitive).
     * 
     * 3. While chars are equal (considering the case sensitivity setting),
     *    advance in both URLs and output the chars. If not equal or one 
     *    string reaches end of string or end of level then return empty string.
     * 
     * 4. If the port part is present and equal in both URLs then it is copied
     *    to the output also, otherwise it is skipped.
     * 
     * 5. While current level less than or equal to levelDepth repeat steps
     *    3 and 4. 
     *
     * @param urls the set of URLs (strings)
     * 
     * @return the common substring
     * 
     * @throws NullPointerException if the argument is null
     * @throws IllegalArgumentException if the set contains something else
     * than Strings (including nulls)
     */
    public String getCommonPart(Set urls) throws NullPointerException, IllegalArgumentException {
        return null;
    }
    
}
