package com.topcoder.web.urlhistory.data;

import java.util.Map;
import java.util.Set;

/**
 * The users, groups and URLs have common features. All three have ids
 * and user defined attributes. The AttributeInfo class is the superclass
 * of the UserInfo, GroupInfo and URLInfo and it defines these common
 * features. 
 * 
 * This class follows the data value object pattern. This class only defines
 * some attributes and accessor methods for them. The subclasses of this abstract
 * class will be used to transfer data between the user, the managers (business 
 * logic layer) and the persistence layer.
 */
public abstract class AttributeInfo {
    
    /**
     * The unique id of the object.
     * This is 0 if the user created the instance and it isn't written to persistence yet
     * and a non-zero value if the instance is created by the persistence implementation.
     * If it is 0, it will be set to a real id value in the persistence
     * create methods, after it is written to persistence.
     * 
     * Generally it can be said that non-zero means the element is stored in the persistent
     * storage.
     * 
     * Set in the constructor taking the id if created by the persistence layer or
     * set in its setter by the persistence layer, if created by the user. 
     */
    private long id = 0;
    
    /**
     * The map of extended attributes. It maps names (Strings) to values (Strings).
     * A HashMap is recommended here. This attribute is initialized in one of
     * the constructors, to either an empty map or a copy of the map given as
     * parameter. There are several accesor methods that change it afterwards.
     */
    private Map attributes = null;
    
    /**
     * Default constructor. It is supposed to be called by the subclasses of this class,
     * from their public constructors.
     * 
     * - the id remains 0
     * - set attributes to an empty map (HashMap)
     */
    protected AttributeInfo() {
    }
    
    /**
     * Constructor. It is supposed to be called by the subclasses of this class from
     * their package private constructors, when invoked by the persistence layer.
     * 
     * - copy the id param to the id attribute
     * - set attributes to a HashMap created as a copy of the given map
     *
     * @param id the id of the object
     * @param attributes the map with the attributes
     * 
     * @throws NullPointerException if attributes is null
     * @throws IllegalArgumentException if id is 0 or if the given map contains
     * something else than strings (including nulls) either as keys or as values 
     */
    protected AttributeInfo(long id, Map attributes) throws NullPointerException, IllegalArgumentException {        
    }
    
    /**
     * Getter for the id field.
     * 
     * - return id
     *
     * @return returns the value of id field
     */
    public long getId() {
        return id;
    }
    
    /**
     * Setter for the id field. Used by the persistence layer to
     * store the id after storing the new element in the database.
     * 
     * - set id to the given value
     *
     * @param id the new id
     */
    protected void setId(long id) {
    }

    // ---
    
    /**
     * Sets the value of an attribute in the attributes map. If the
     * attribute already exists, the value is replaced.
     * 
     * - put in the attributes map the (name, value) pair
     *
     * @param name the name of the attribute
     * @param value the value of the attribute
     *
     * @throws NullPointerException if any argument is null
     */
    public void setAttribute(String name, String value) throws NullPointerException {
    }
    
    /**
     * Removes an attribute. No action is performed if the attribute does not exist.
     * 
     * - remove the name key from the map
     * - return whether the attribute existed
     *
     * @param name the name of the attribute
     * 
     * @return whether the attribute existed
     *
     * @throws NullPointerException if the argument is null
     */
    public boolean removeAttribute(String name) throws NullPointerException {
        return false;
    }
    
    /**
     * Clears all attributes.
     * 
     * - clear the attributes map
     */
    public void clearAttributes() {
    }
    
    /**
     * Gets the list of defined attribute names.
     * 
     * - return a copy of the key set of the attributes map
     *
     * @return the list of attribute names 
     */
    public Set getAttributeNames() {
        return null;
    }
    
    /**
     * Gets the value of an attribute.
     * 
     * - return the value of the given key or null if the key doesn't exist
     *
     * @param name the name of attribute to get
     * 
     * @return the value of the attribute (null if it doesn't exist)
     *  
     * @throws NullPointerException if the argument is null
     */
    public String getAttribute(String name) throws NullPointerException {
        return null;
    }
        
}
