package com.topcoder.web.urlhistory.data;

import java.util.Map;


/**
 * The groups and URLs have some common features, besides those defined in the 
 * AttributeInfo class. Both have a name and the never expire and black listed flags and
 * are own by a user (specified by the user id).
 * The CommonInfo class is the superclass GroupInfo and URLInfo and it defines these
 * common features.
 * 
 * The black list flag and never expire flags are implemented the same using
 * an indicator that can have 3 values. The YES value means the object is
 * black listed or never expires, the NO value means the opposite. The third
 * value is DEFAULT. It means the object takes the value from the parent
 * (and in case the parent is DEFAULT, from the parent of the parent and so on).
 * If all parents have DEFAULT value then the value NO is considered.
 * 
 * In the context of nested groups, this 3-value flagging technique brings great
 * power in the ability to define black listing and never expire rules. One can
 * flag a group as black listed, but mark one of its subgroups as not black listed.
 * Even more, even if the subgroup is not black listed, any of its URLs can still
 * be marked as black listed. 
 * 
 * This class follows the data value object pattern. This class only defines
 * some attributes and accessor methods for them. The subclasses of this abstract
 * class will be used to transfer data between the user, the managers (business 
 * logic layer) and the persistence layer.
 */
public abstract class CommonInfo extends AttributeInfo {
    
    /**
     * The unique id of the user owning the object.
     * This is 0 if the user created the instance and it isn't written to persistence yet
     * and a non-zero value if the instance is created by the persistence implementation.
     * If it is 0, it will be set to a real id value in the persistence
     * create methods, after it is written to persistence.
     * 
     * Set in the constructor taking the user id if created by the persistence layer or
     * set in its setter by the persistence layer, if created by the user.
     * 
     * The purpose of this attribute is to enabled sanity checks when
     * the user tries to set the parent-child relationships between
     * objects belonging to different users.
     */
    private long userId = 0;
    
    /**
     * The descriptive name of the group or URL.
     * Initialized in one of the constructors and modifiable using its setter.
     */
    private String name = null;
    
    /**
     * Indicates the never expire status of this object (see the class description).
     * The default value can be changed in the second constructor or in the setter.
     */
    private Flag neverExpire = Flag.DEFAULT;
    
    /**
     * Indicates the black listed status of this object (see the class description).
     * The default value can be changed in the second constructor or in the setter.
     */
    private Flag blackListed = Flag.DEFAULT;
    
    /**
     * Constructor. It is supposed to be called by the subclasses of this class,
     * from their public constructors.
     *
     * - the name is set
     *
     * @param name the name of the object
     * 
     * @throws NullPointerException if the argument is null
     */
    protected CommonInfo(String name) throws NullPointerException {
    }
    
    /**     
     * Constructor. It is supposed to be called by the subclasses of this class from
     * their package private constructors, when invoked by the persistence layer.
     * 
     * - calls super constructor with the id and attributes
     * - sets the name, neverExpire and blackListed attributes 
     *
     * @param id the id of the object
     * @param userId the user id of the user owning the object 
     * @param name the name of the object
     * @param attributes the map with the attributes
     * @param neverExpire the never expire flag
     * @param blackListed the black listed flag
     * 
     * @throws NullPointerException if name, attributes or the flags are null
     * @throws IllegalArgumentException if id is 0 or if the given map contains
     * something else than strings (including nulls) either as keys or as values 
     */
    protected CommonInfo(long id, long userId, String name, Map attributes, Flag neverExpire, Flag blackListed) throws NullPointerException, IllegalArgumentException {
        super(id, attributes);
    }
    
    // ---
    
    /**
     * Getter for the userId field.
     *
     * @return returns the value of userId field
     */
    public long getUserId() {
        return userId;
    }
    /**
     * Setter for the userId field. Called by the persistence layer to set the user
     * id after the object is persisted.
     *
     * @param userId the value for the userId field
     */
    protected void setUserId(long userId) {
        this.userId = userId;
    }

    
    // ---

    /**
     * Getter for the name field.
     *
     * @return returns the value of name field
     */
    public String getName() {
        return name;
    }
    
    /**
     * Setter for the name field.
     *
     * @param name the value for the name field
     *
     * @throws NullPointerException if the argument is null
     */
    public void setName(String name) throws NullPointerException {
        this.name = name;
    }
    
    // ---
    
    /**
     * Getter for the blackListed field.
     *
     * @return returns the value of blackListed field
     */
    public Flag getBlackListed() {
        return blackListed;
    }
    
    /**
     * Setter for the blackListed field.
     *
     * @param blackListed the value for the blackListed field
     *
     * @throws NullPointerException if the argument is null
     */
    public void setBlackListed(Flag blackListed) throws NullPointerException {
        this.blackListed = blackListed;
    }
    
    /**
     * Getter for the neverExpire field.
     *
     * @return returns the value of neverExpire field
     */
    public Flag getNeverExpire() {
        return neverExpire;
    }
    
    /**
     * Setter for the neverExpire field.
     *
     * @param neverExpire the value for the neverExpire field
     *
     * @throws NullPointerException if the argument is null
     */
    public void setNeverExpire(Flag neverExpire) throws NullPointerException {
        this.neverExpire = neverExpire;
    }
    
}
