package com.topcoder.web.urlhistory.data;

import java.util.Date;

/**
 * The statistics classes are used to return to the user statistical information
 * about groups and URLs.
 * 
 * The groups and URLs statistics have common fields. Both have visit count,
 * last visit date and personal rating. This class is a superclass for the
 * GroupStatistics and URLStatistics classes that define these common fields,
 * along with the appropriate getters.
 * 
 * Note that both group and URL statistics may refer to several actual URLs.
 * That's because group statistics aggregate statistics for all URLs that have
 * them as parent. URL statistics may aggregate several URLs too, because of
 * the query / sort based on comparators requirement.
 * 
 * This class follows the data value object pattern. This class only defines
 * some attributes and accessor methods for them. The subclasses of this abstract
 * class will be used to transfer data between the persistence layer, business logic
 * and the user.
 */
public class CommonStatistics {

    /**
     * The visit count. For URLs this means the number of visits to the URL
     * that are stored in persistence. For groups and for URLs that aggregate
     * more individual URLs, this is defined as the sum of the individual
     * visit counts.
     *  
     * Set in the constructor.
     */
    private int visitCount = 0;
    
    /**
     * The last visit date. For URLs this means the last visit to the URL
     * that is stored in persistence. For groups and for URLs that aggregate
     * more individual URLs, this is defined as the last visit for all the individual
     * URLs (the max of the individual last visit dates).
     *  
     * Set in the constructor.
     */
    private Date lastVisit = null;
    
    /**
     * The personal rating. For URLs this means the personal rating number
     * that is stored in persistence. For groups and for URLs that aggregate
     * more individual URLs, this is defined as the average personal rating number
     * of the individual aggregated URLs.
     *  
     * Set in the constructor.
     */
    private int personalRating = 0;
    
    /**
     * Protected constructor. Called by the subclasses.
     * 
     * - sets the 3 attributes
     *
     * @param visitCount the visit count
     * @param lastVisit the last visit
     * @param personalRating the personal rating 
     * 
     * @throws NullPointerException if the Date is null  
     */
    protected CommonStatistics(int visitCount, Date lastVisit, int personalRating) throws NullPointerException {            
    }
    
    /**
     * Getter for the visitCount field.
     *
     * @return returns the value of visitCount field
     */
    public int getVisitCount() {
        return visitCount;
    }

    /**
     * Getter for the lastVisit field.
     *
     * @return returns the value of lastVisit field
     */
    public Date getLastVisit() {
        return lastVisit;
    }
    
    /**
     * Getter for the personalRating field.
     *
     * @return returns the value of personalRating field
     */
    public int getPersonalRating() {
        return personalRating;
    }
    
}
