package com.topcoder.web.urlhistory.data;

import com.topcoder.util.collection.typesafeenum.Enum;

/**
 * Enumeration used for the values of the black listed and expire flags. It is
 * implemented as a subclass of the Enum class from the Type Safe Enumeration 
 * component. 
 * 
 * The black list flag and never expire flags are implemented the same using
 * an indicator that can have 3 values. The YES value means the object is
 * black listed or never expires, the NO value means the opposite. The third
 * value is DEFAULT. It means the object takes the value from the parent
 * (and in case the parent is DEFAULT, from the parent of the parent and so on).
 * If all parents have DEFAULT value then the value NO is considered.
 * 
 * In the context of nested groups, this 3-value flagging technique brings great
 * power in the ability to define black listing and never expire rules. One can
 * flag a group as black listed, but mark one of its subgroups as not black listed.
 * Even more, even if the subgroup is not black listed, any of its URLs can still
 * be marked as black listed. 
 */
public class Flag extends Enum {
    
    /**
     * The DEFAULT value means the value is taken from the parent group.
     * If the parent is DEFAULT too, then the parent of the parent is consulted and so on.
     * If all parents are DEFAULT (or no parents exist), then NO is considered as value. 
     */
    public static final Flag DEFAULT = new Flag(0);
        
    /**
     * The YES value means the group / URL is black listed / never expires. 
     */
    public static final Flag YES = new Flag(1);
    
    /**
     * The NO value means the group / URL isn't black listed / expires as usual. 
     */    
    public static final Flag NO = new Flag(2);
    
    /**
     * The value of the flag. The persistence implements this flag as
     * a short integer taking 0, 1 and 2 as values. This value provides
     * easier comparisons.
     */
    private int value = 0;
    
    /**
     * Private constructor. Prevents outside instantation.
     *
     * @param value the value of the flag
     */
    private Flag(int value) {
    }
    
    /**
     * Gets the value of the flag.
     *
     * @return the value of the flag
     */
    public int getValue() {
        return value;
    }
    
    /**
     * Gets the value of the flag as string (converts the number to string).
     * 
     * @return the value of the flag as string
     */
    public String toString() {
        return null; 
    }

}
