package com.topcoder.web.urlhistory.data;

import java.util.Map;

/**
 * This class represents a group. A group has several attributes such as
 * id, name, user defined attributes, never expire flag, black list flag and
 * the id of the parent.
 * 
 * Groups can be nested. The parentId attributes defines the id of the parent
 * group (if any). The GroupManager class uses this attribute to retrieve the
 * parent from persistence. It also offers a method to retrieve the children of
 * a group.
 * 
 * This class follows the data value object pattern. This class defines only
 * the parentId attribute. All the other attributes are implemented in the superclass.
 */
public class GroupInfo extends CommonInfo {

    /**
     * The id of the parent group of this group.
     * It can be 0 if it is a root group.
     * Set either in the second constructor when called by the persistence layer,
     * or in the setter, by the persistence layer, when the user creates
     * a new subgroup.
     */
    private long parentId = 0;
    
    /**
     * Constructor. It is supposed to be called by the user, when he
     * wants to define a new group to be created.
     *
     * - the name is set
     *
     * @param name the name of the group
     * 
     * @throws NullPointerException if the argument is null
     */
    public GroupInfo(String name) throws NullPointerException {
        super(name);
    }
    
    /**     
     * Constructor. It is supposed to be called by the subclasses of this class from
     * their package private constructors, when invoked by the persistence layer.
     * 
     * - calls super constructor with the id, name, attributes and the two flags
     * - sets the parentId 
     *
     * @param id the id of the object
     * @param userId the user id of the user owning the group
     * @param name the name of the object
     * @param attributes the map with the attributes
     * @param neverExpire the never expire flag
     * @param blackListed the black listed flag
     * @param parentId the id of the parent group (0 means root group)
     * 
     * @throws NullPointerException if name, attributes or the flags are null
     * @throws IllegalArgumentException if id is 0 or if the given map contains
     * something else than strings (including nulls) either as keys or as values 
     */
    protected GroupInfo(long id, long userId, String name, Map attributes, Flag neverExpire, Flag blackListed, long parentId) throws NullPointerException, IllegalArgumentException {
        super(id, userId, name, attributes, neverExpire, blackListed);
    }
    
    /**
     * Getter for the parentId field.
     *
     * @return returns the value of parentId field
     */
    public long getParentId() {
        return parentId;
    }        
        
    /**
     * Setter for the parentId field. Can be called by the persistence layer when
     * creating this group as subgroup of another group.
     *
     * @param parentId the value for the parentId field
     */
    protected void setParentId(long parentId) {
        this.parentId = parentId;
    }
    
}
