package com.topcoder.web.urlhistory.data;

import java.util.Map;
import java.util.Set;

/**
 * This class represents information about a URL. A URL has several attributes such as
 * id, name, user defined attributes, never expire flag, black list flag,
 * actual URL text and the set of group which it is a member of.
 * 
 * This class follows the data value object pattern. This class defines only
 * the URL and parent groups attributes and accessor method for them. 
 * All the other attributes are implemented in the CommonInfo superclass.
 */
public class URLInfo extends CommonInfo {
    
    /**
     * The actual URL string.
     * Initialized in the constructors and can be changed using the setter.
     */
    private String url = null;
    
    /**
     * The set of groups (GroupInfo) this URL is a member of (parent groups).
     * This is initialized with an empty HashSet in the public constructor
     * or with a HashSet copy of the given set in the other one.
     */
    private Set groups = null;
    
    /**
     * Constructor. It is supposed to be called by the user, when he
     * wants to define a new URL to be created.
     *
     * - the name and url are set
     * - groups is set to an empty HashSet
     *
     * @param name the name of the URL
     * @param url the actual url string of the URL 
     * 
     * @throws NullPointerException if any argument is null
     */
    public URLInfo(String name, String url) throws NullPointerException {
        super(name);
    }
    
    /**     
     * Constructor. It is supposed to be called by the subclasses of this class from
     * their package private constructors, when invoked by the persistence layer.
     * 
     * - calls super constructor with the id, name, attributes and the two flags
     * - sets the url
     * - sets groups as a HashSet copy of the given group set
     *
     * @param id the id of the object
     * @param userId the user id of the user owning the object
     * @param name the name of the object
     * @param attributes the map with the attributes
     * @param neverExpire the never expire flag
     * @param blackListed the black listed flag
     * @param url the actual url string of the URL
     * @param groups the parent groups (set of GroupInfo)
     * 
     * @throws NullPointerException if name, attributes, flags, url or groups are null
     * @throws IllegalArgumentException if id is 0 or if the given map contains
     * something else than strings (including nulls) either as keys or as values,
     * or if the groups set contains something else that GroupInfo (including nulls)
     */
    protected URLInfo(long id, long userId, String name, Map attributes, Flag neverExpire, Flag blackListed, String url, Set groups) throws NullPointerException, IllegalArgumentException {
        super(id, userId, name, attributes, neverExpire, blackListed);
    }
    
    // ---

    /**
     * Getter for the url field.
     *
     * @return returns the value of url field
     */
    public String getUrl() {
        return url;
    }
    
    /**
     * Setter for the url field.
     *
     * @param url the value for the url field
     */
    public void setUrl(String url) {
        this.url = url;
    }
    
    // ---
    
    /**
     * Adds a parent group to the set.
     *
     * @param group the group
     *
     * @throws NullPointerException if the argument is null
     */
    public void addGroup(GroupInfo group) throws NullPointerException {
    }
    
    /**
     * Removes a parent group from the set.
     * 
     * No action is performed if the group isn't a parent group of this URL.
     * 
     * @param group the group
     * 
     * @return whether the group was a parent
     *
     * @throws NullPointerException if the argument is null
     */
    public boolean removeGroup(GroupInfo group) throws NullPointerException {
        return false;
    }
    
    /**
     * Clears all parent groups (clears the set).
     */
    public void clearGroups() {
    }
    
    /**
     * Gets the set of parent groups (a copy).
     * 
     * @return the set of parent groups 
     */
    public Set getGroups() {
        return null;
    }
    
}
