package com.topcoder.web.urlhistory.data;

import java.util.Map;

/**
 * This class represents information about a user. This isn't a very important
 * entity in this design. The user is given by a UserProfile and the toString value
 * of this UserProfile is used as key in the persistent storage in order to
 * keep URL and group information separate for each user. The only reason
 * for this class is to allow URL history expiration options to be set separately
 * for each user.
 * 
 * Since the expiration options are different for each user, they must be stored
 * in the persistence storage and manipulation methods must be provided for them.
 * The expiration options are implemented using the user defined attributes specific
 * to groups and URLs. This allows us to set different expiration options without
 * having to hardcode special fields for them. 
 * 
 * The persistence implementations are responsible for enforcing the storage
 * limits because they are persistence specific (for example age based, max entry 
 * count based or max size based). They are responsible to define attribute names 
 * that can be used to define different expiration options (see the DatabasePersistence
 * class for details).  
 * 
 * In addition to providing expiration options, there is support to define any
 * other attributes. These attributes may be used in the future to configure
 * anything else regarding user preferences. 
 * 
 * This class follows the data value object pattern. All the attributes are 
 * implemented in the AttributeInfo superclass. The purpose of defining this
 * class is only to have a type specialized for representing users.
 * AttributeInfo has exactly the same capabilities, but using it is not type
 * safe and a bit confusing for the user.
 */
public class UserInfo extends AttributeInfo {
    
    /**
     * Constructor. This is called by the used when creating a new user entry.
     * Note that all the manager methods automatically create a user entry if
     * none exists. This constructor and the GeneralManager.createUser method
     * should only be used when the user simply wants to set expiration options.  
     */
    public UserInfo() {
        super();
    }
    
    /**
     * Constructor. It is supposed to be called by the persistence layer when
     * retrieving user information from persistence.
     * 
     * - simply call the super constructor
     *
     * @param id the id of the object
     * @param attributes the map with the attributes
     * 
     * @throws NullPointerException if attributes is null
     * @throws IllegalArgumentException if id is 0 or if the given map contains
     * something else than strings (including nulls) either as keys or as values 
     */
    protected UserInfo(long id, Map attributes) throws NullPointerException, IllegalArgumentException {
        super(id, attributes);
    }

}
