/*
 * Created on Oct 26, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package com.topcoder.web.urlhistory.persistence;

import java.util.Date;
import java.util.Set;

import com.topcoder.user.profile.UserProfile;
import com.topcoder.web.urlhistory.ConfigurationException;
import com.topcoder.web.urlhistory.data.GroupInfo;
import com.topcoder.web.urlhistory.data.URLInfo;
import com.topcoder.web.urlhistory.data.URLStatistics;
import com.topcoder.web.urlhistory.data.UserInfo;

/**
 * This class is a concrete implementation of the PersistenceStrategy interface 
 * that uses a database as persistence storage.
 * 
 * The database details are taken from a configuration manager namespace.
 * This namespace is given by the ForumManager class when it calls the
 * constructor for this class through reflection. The configuration includes
 * the name of the JDBC driver, the connection string, database name, user
 * name and password. Alternatively the database may be configured using JNDI
 * (usefull if deployed inside a application container).
 * 
 * This class implements the expiration logic directly. That is because expiration
 * checking and purging is too persistence specific to be able to abstract it
 * and make it general to all persistence implementations. It may be possible
 * to do it but the performance may suffer because of it.
 * 
 * The age based expiration check is performed using the Job Scheduling component
 * to set up a job that runs from time to time. The maximum entry count and
 * maximum size checks are performed when create URL visits.
 */
public class DatabasePersistence implements PersistenceStrategy {
    
    /**
     * The expiration age constant indicates the user attribute name
     * that indicates the number of days before a URL history entry expires.
     * 
     * The expiration check is done by a separate job that is started by
     * the constructors. This jobs runs from time to time (the period is
     * indicated in the config file) and it removes the expired entries.
     */
    public static final String EXPIRATION_AGE_KEY = "expiration_age";
    
    /**
     * The expiration max entries constant indicates the user attribute name
     * that indicates the maximum history entry count for a user.
     * 
     * This check is performed when the createVisit method is called.
     */
    public static final String EXPIRATION_MAX_ENTRIES_KEY = "expiration_max_entries";
    
    /**
     * The expiration max entries constant indicates the user attribute name
     * that indicates the maximum history size for a user.
     * 
     * The size of one history entry is roughly 128 bits, 16 bytes (the URL id
     * is a bigint that takes 64 bits and the Date should not be less that 64 bits).
     * 
     * This check is performed when the createVisit method is called. 
     */
    public static final String EXPIRATION_MAX_SIZE_KEY = "expiration_max_size";
    
    /**
     * Constant for the name of the ID Generator sequence to use.
     * The same ID Generator sequence is used for all objects.
     */
    private static final String ID_SEQUENCE_NAME = "URL_HISTORY";

    /** 
     * Protected constructor.
     * Called through reflection by the GeneralManager class.
     * 
     * The constructor loads the JDBC details of the database from the namespace
     * (driver class, database name, user and password). In case these properties are
     * missing the constructor code looks for the JNDI name for the DataSource. This
     * allows both types of configurations to be used.
     * 
     * The db_expiration_check_interval property is read also and a Scheduler is created
     * using it. The job scheduler that runs at this interval is set up using:
     * 
     * Scheduler scheduler = new Scheduler(unique_name_to_this_app);
     * scheduler.addJob(new Job("some_name", new GregorianCalendar(), null, interval_in_seconds, Calendar.SECOND,
     *                Scheduler.JOB_TYPE_JAVA_CLASS, some_inner_class.class.getName()));
     * scheduler.start();
     * 
     * Note that history entries for URLs marked as blacklisted or never expiring are not
     * taken into account in this check.
     * 
     * @param namespace the namespace
     * 
     * @throws NullPointerException if the argument is null
     * @throws ConfigurationException wraps exceptions thrown by the Configuration 
     * Manager if an error occurs while working with the config file
     * @throws PersistenceException wraps any exceptions occuring while trying to
     * obtain the database connection or the DataSource (SQL exception, JNDI
     * lookup exceptions)
     */
    protected DatabasePersistence(String namespace) throws NullPointerException, ConfigurationException, PersistenceException {
    }

    /**
     * Creates a user in the persistent storage.
     *
     * @param userProfile the user profile
     * @param user the user info object
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createUser(UserProfile userProfile, UserInfo user) throws NullPointerException, PersistenceException { 
    }
    
    /** 
     * Updates a user in the persistent storage.
     *
     * @param user the user info object
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void updateUser(UserInfo user) throws NullPointerException, PersistenceException { 
    }
    
    /** 
     * Deletes a user from the persistent storage.
     *
     * @param user the user info object
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void deleteUser(UserInfo user) throws NullPointerException, PersistenceException { 
    }
    
    /**
     * Reads a user from the persistent storage.
     *
     * @param user the user profile
     * 
     * @return the user info for the user
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public UserInfo readUser(UserProfile user) throws NullPointerException, PersistenceException { 
        return null;
    }
    
    /**
     * Reads all user from the persistent storage.
     * 
     * @return set of UserInfo
     *
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readAllUsers() throws NullPointerException, PersistenceException { 
        return null;
    }
    
    // ---
    
    /**
     * Creates a group / subgroup in the persistent storage.
     * If the parent is null then a group is created (root group), otherwise
     * a subgroup of the parent is created.
     * 
     * @param user the user profile (can be null if parent isn't null)
     * @param group the group
     * @param parent the parent group (can be null is user isn't null)
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createGroup(UserProfile user, GroupInfo group, GroupInfo parent) throws NullPointerException, PersistenceException { 
    }
    
    /**
     * Updates a group in the persistent storage.
     *
     * @param group the group
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void updateGroup(GroupInfo group) throws NullPointerException, PersistenceException { 
    }

    /**
     * Deletes a group from the persistent storage.
     *
     * @param group the group
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void deleteGroup(GroupInfo group) throws NullPointerException, PersistenceException { 
    }
          
    /**
     * Reads group from persistence given the id of the group.
     *
     * @param id the id of the group
     * @return the group info
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public GroupInfo readGroup(long id) throws PersistenceException { 
        return null;
    }
    
    /**
     * Reads the subgroups of a group. If the group is null, the root groups
     * are read.
     *
     * @param group the group (null allowed)
     * 
     * @return the set of groups / subgroups (set of GroupInfo)
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readSubgroups(GroupInfo group) throws PersistenceException { 
        return null;
    }
    
    /**
     * Creates a URL in the persistent storage.
     * 
     * @param user the user profile (can be null if parent isn't null)
     * @param url the URLInfo
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createURL(UserProfile user, URLInfo url) throws NullPointerException, PersistenceException { 
    }
    
    /**
     * Updates a URL in the persistent storage.
     *
     * @param url the URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void updateURL(URLInfo url) throws NullPointerException, PersistenceException { 
    }
    
    /**
     * Deletes a URL from the persistent storage.
     *
     * @param url the URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void deleteURL(URLInfo url) throws NullPointerException, PersistenceException { 
    }
    
    /**
     * Read URL from persistence given the id of the URL.
     *
     * @param id the id of the URL
     * 
     * @return the URL info
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public URLInfo readURL(long id) throws PersistenceException { 
        return null;
    }
    
    /**
     * Reads URL from persistence given the user profile and the URL string.
     *
     * @param user the user profile
     * @param url the URL string
     * 
     * @return the URL info
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public URLInfo readURL(UserProfile user, String url) throws NullPointerException, PersistenceException { 
        return null;
    }
    
    /**
     * Reads the URLs within a group. If the group is null, the root URLs are returned
     * (without any parent group).
     *
     * @param group the group (null allowed)
     * 
     * @return the set of URLs (set of URLInfo)
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readURLs(GroupInfo group) throws PersistenceException { 
        return null;
    }
    
    /**
     * Reads all URLs for a user.
     *
     * @param user the user profile
     * 
     * @return the set of URLs (set of URLInfo)
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readAllURLs(UserProfile user) throws NullPointerException, PersistenceException { 
        return null;
    }
    
    /**
     * Creates a visit record in the history for the given URL, with the given time.
     * 
     * The database is checked for expiration based on the user defined max entry count
     * and size. The 2 values are retrieved from the database and then the check is performed.
     * If it is the case, the oldest history entries are removed.
     * 
     * Note that history entries for URLs marked as blacklisted or never expiring are not
     * taken into account in this check. 
     *
     * @param url the url
     * @param date the date
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createVisit(URLInfo url, Date date) throws NullPointerException, PersistenceException { 
    }

    /** 
     * Gets a URLStatistics object for the given URL, that contains information
     * such as last visit and visit counts.
     *
     * @param url the url
     * 
     * @return the URLStatistics for the given URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public URLStatistics getVisitStatistic(URLInfo url) throws NullPointerException, PersistenceException { 
        return null;
    }
    
    /**
     * Gets a URLStatistics objects for the given user, that contains information
     * such as last visit and visit counts for each URL.
     * 
     * This raw result is then processed by the business logic.
     *
     * @param user the user
     * 
     * @return the visit statistics for the user (set of URLStatistics)
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set getVisitStatistics(UserProfile user) throws NullPointerException, PersistenceException {
        return null;
    }

}
