package com.topcoder.web.urlhistory.persistence;

import java.util.Date;
import java.util.Set;

import com.topcoder.user.profile.UserProfile;
import com.topcoder.web.urlhistory.data.GroupInfo;
import com.topcoder.web.urlhistory.data.URLInfo;
import com.topcoder.web.urlhistory.data.URLStatistics;
import com.topcoder.web.urlhistory.data.UserInfo;

/**
 * This interface provides a layer of abstraction for the persistence of the 
 * URL History objects. It is essentially a class that provides the standard 
 * CRUD operations (create, read, update, delete) for the entities used by 
 * this component (user, group, URL).
 * 
 * The methods exposes by this interface are very raw (it would be hard for a user
 * to use them to obtain the forum functionality). They are aimed to an efficient 
 * database implementation (using INSERT, SELECT, UPDATE and DELETE statements) but
 * other storage technologies can be used just as well (such as XML).
 * 
 * The implementations of this class must provide a constructor taking one string
 * parameter (the config manager namespace). This constructor will be called by
 * the ForumFactory class through reflection.
 */
public interface PersistenceStrategy {
    
    /**
     * Creates a user in the persistent storage.
     *
     * @param userProfile the user profile
     * @param user the user info object
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createUser(UserProfile userProfile, UserInfo user) throws NullPointerException, PersistenceException;
    
    /** 
     * Updates a user in the persistent storage.
     *
     * @param user the user info object
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void updateUser(UserInfo user) throws NullPointerException, PersistenceException;
    
    /** 
     * Deletes a user from the persistent storage.
     *
     * @param user the user info object
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void deleteUser(UserInfo user) throws NullPointerException, PersistenceException;
    
    /**
     * Reads a user from the persistent storage.
     *
     * @param user the user profile
     * 
     * @return the user info for the user
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public UserInfo readUser(UserProfile user) throws NullPointerException, PersistenceException;
    
    /**
     * Reads all user from the persistent storage.
     * 
     * @return set of UserInfo
     *
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readAllUsers() throws NullPointerException, PersistenceException;
    
    // ---
    
    /**
     * Creates a group / subgroup in the persistent storage.
     * If the parent is null then a group is created (root group), otherwise
     * a subgroup of the parent is created.
     * 
     * @param user the user profile (can be null if parent isn't null)
     * @param group the group
     * @param parent the parent group (can be null is user isn't null)
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createGroup(UserProfile user, GroupInfo group, GroupInfo parent) throws NullPointerException, PersistenceException;
    
    /**
     * Updates a group in the persistent storage.
     *
     * @param group the group
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void updateGroup(GroupInfo group) throws NullPointerException, PersistenceException;

    /**
     * Deletes a group from the persistent storage.
     *
     * @param group the group
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void deleteGroup(GroupInfo group) throws NullPointerException, PersistenceException;
          
    /**
     * Reads group from persistence given the id of the group.
     *
     * @param id the id of the group
     * @return the group info
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public GroupInfo readGroup(long id) throws PersistenceException;
    
    /**
     * Reads the subgroups of a group. If the group is null, the root groups
     * are read.
     *
     * @param group the group (null allowed)
     * 
     * @return the set of groups / subgroups (set of GroupInfo)
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readSubgroups(GroupInfo group) throws PersistenceException;
    
    /**
     * Creates a URL in the persistent storage.
     * 
     * @param user the user profile (can be null if parent isn't null)
     * @param url the URLInfo
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createURL(UserProfile user, URLInfo url) throws NullPointerException, PersistenceException;
    
    /**
     * Updates a URL in the persistent storage.
     *
     * @param url the URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void updateURL(URLInfo url) throws NullPointerException, PersistenceException;
    
    /**
     * Deletes a URL from the persistent storage.
     *
     * @param url the URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void deleteURL(URLInfo url) throws NullPointerException, PersistenceException;
    
    /**
     * Read URL from persistence given the id of the URL.
     *
     * @param id the id of the URL
     * 
     * @return the URL info
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public URLInfo readURL(long id) throws PersistenceException;
    
    /**
     * Reads URL from persistence given the user profile and the URL string.
     *
     * @param user the user profile
     * @param url the URL string
     * 
     * @return the URL info
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public URLInfo readURL(UserProfile user, String url) throws NullPointerException, PersistenceException;
    
    /**
     * Reads the URLs within a group. If the group is null, the root URLs are returned
     * (without any parent group).
     *
     * @param group the group (null allowed)
     * 
     * @return the set of URLs (set of URLInfo)
     * 
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readURLs(GroupInfo group) throws PersistenceException;
    
    /**
     * Reads all URLs for a user.
     *
     * @param user the user profile
     * 
     * @return the set of URLs (set of URLInfo)
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set readAllURLs(UserProfile user) throws NullPointerException, PersistenceException;
    
    /**
     * Creates a visit record in the history for the given URL, with the given time.
     *
     * @param url the url
     * @param date the date
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public void createVisit(URLInfo url, Date date) throws NullPointerException, PersistenceException;

    /** 
     * Gets a URLStatistics object for the given URL, that contains information
     * such as last visit and visit counts. 
     *
     * @param url the url
     * 
     * @return the URLStatistics for the given URL
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public URLStatistics getVisitStatistic(URLInfo url) throws NullPointerException, PersistenceException;
    
    /**
     * Gets a URLStatistics objects for the given user, that contains information
     * such as last visit and visit counts for each URL.
     * 
     * This raw result is then processed by the business logic.
     *
     * @param user the user
     * 
     * @return the visit statistics for the user (set of URLStatistics)
     * 
     * @throws NullPointerException if any argument is null
     * @throws PersistenceException wraps a persistence implementation specific exception
     * (such as SQL exception or XML exception) 
     */
    public Set getVisitStatistics(UserProfile user) throws NullPointerException, PersistenceException;
    
}
